///////////////////////////////////////////////////
//
// Task Binary Tree
// File: output.cpp
//
// Contents: The printf member functions of the
//           classes tree and branch
//
// This file is fully implemented. If you did not
// edit it, you also do not need to mail it in.
// Skeleton function by patrick@magma-da.com
///////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <iostream.h>
#include "tree.h"


///////////////////////////////////////////////////
//
// Prints the words in alphabetical order.
// Wrapper funtion on class tree: calls
// branch::print_word_alphabetically() on the _root branch.
//
void tree::print_words_alphabetically(void) const
{
   if(_root != 0)
      _root->print_words_alphabetically();
}


///////////////////////////////////////////////////
//
// Prints the number of words (=branches) contained in the tree.
// Wrapper funtion on class tree: calls
// branch::count() on the _root branch.
//
void tree::print_count(void) const
{
   int cnt = 0;
   if(_root)
      cnt = _root->count();
   printf("The tree contains %d branch%s\n", cnt, cnt == 1 ? "" : "es");
}

///////////////////////////////////////////////////
//
// Prints the maximum height of the tree.
// Wrapper funtion on class tree: calls
// branch::height() on the _root branch.
//
void tree::print_height(void) const
{
   int height = 0;
   if(_root != 0)
      height = _root->height();

   printf("The height of the tree is %d\n", height);
}

///////////////////////////////////////////////////
//
// Prints whether a word exists in the tree.
// Wrapper funtion on class tree: calls
// branch::find(word) on the _root branch.
//
void tree::print_whether_word_exists(const char * const word) const
{
   if(_root != 0 && _root->find(word) != 0)
      printf("A branch containing '%s' does indeed exist in the tree.\n", word);
   else
      printf("A branch containing '%s' does not exist in the tree.\n", word);
}


/* * * * * * * * *
 *
 * Help routine to print 'count' spaces recursively
 *
 * * * * * * * * */
static void print_spaces(const int count)
{
   if(count <= 0)
      return;
   printf(" ");
   print_spaces(count-1);
}

/* * * * * * * * *
 *
 * recursively prints a single level of the tree
 *
 * * * * * * * * */
void branch::printlevel(const int level, const int width) const
{
   const int half_width = width / 2;
   if(level == 1) {
      print_spaces(half_width);
      printf("%s",_word);
      print_spaces(half_width - strlen(_word));
   } else {
      if(_left)
         _left->printlevel(level-1, half_width);
      else
         print_spaces(half_width);
      if(_right)
         _right->printlevel(level-1, half_width);
      else
         print_spaces(half_width);
   }
}


/* * * * * * * * *
 *
 * Plots the contents of the tree, with the root node in
 * the top center of the screen.
 * Due to the limited size of a screen, this function will only
 * produce nice output on small trees with short words.
 *
 * * * * * * * * */
void tree::print(void) const
{

   print_count();
   if(_root == 0)
      return;   // tree is empty
   int height = _root->height();
   if(height < 0) {
      cerr << "ERROR: cannot plot tree. You must implement member function branch::height() first. " << endl;
      return;
   }
   for(int i = 1; i <= height; i++) {
      _root->printlevel(i, 64); // print out tree, level by level
      printf("\n");
   }
}
