//////////////////////////////////////////////////////////////////
//
// Task Binary Tree
// File: tree.h
//
// Contents: definitions of tree and branch classes
//
// This file is fully implemented. If you did not
// edit it, you also do not need to mail it in.
// Skeleton functions by patrick@magma-da.com
/////////////////////////////////////////////////////////////////
#include <stdio.h>

class branch;    // forward declaration of the class branch

 ////////////////////////////////////////////////////////////////
//
// The tree class owns a binary tree of words.
// As data member it contains a pointer to the root
// of the tree (which is of type 'branch').
//
class tree {
public:
    tree(void);  // ctor : makes an empty tree
    ~tree(void); // dtor : destroys all branches of the tree

    void         // Empties the tree: destroys all branches it contains
       destroy(void);

    void         // adds a word to the tree
       add(const char * const word);
    void         // removes a word from the tree
       remove(const char * const word);

    void         // Prints all words in alphabetical order
       print_words_alphabetically(void) const;
    void         // Prints whether or not a word exists in the tree
       print_whether_word_exists(const char * const word) const;
    void         // Prints the maximum height of the tree
       print_height(void) const;
    void         // Prints the number of words in the tree
       print_count(void) const;
    void         // Plots the contents of the tree
       print(void) const;

    void         // writes the contents of the tree into a file called fname
       write_into_file(const char * const fname) const;
    void         // reads words from a file called 'fname'
       read_from_file(const char * const fname);
private:
    branch *     // pointer to the root branch of the tree
       _root;    // (if this is 0, the tree is empty).
};


////////////////////////////////////////////////////////////////
//
// The class 'branch' contains the nodes (branches) of a binary
// tree. Each node contains a string and 2 pointers
// to the left and right subtrees.
//
class branch {
public:
    branch(const char * const word);  // ctor : makes a branch containing 'word'
    ~branch(void);                    // dtor : frees the word
    void         // recursively destroys the left and right subtrees and deletes this
       destroy(void);

    void         // adds a word to the subtree at this branch
       add(const char * const word);
    branch *     // removes the branch containing word from the tree
       remove(const char * const word, branch ** ptr_ptr);

    void         // prints the words of this subtree alphabetically
       print_words_alphabetically(void) const;
    branch *     // finds a word. Returns 0 if the word does not exist
       find(const char * const word) const;
    int          // returns the maximum height of the subtree below this branch
       height(void) const;
    int          // returns the number of words in the subtree below this
       count(void) const;
    void         // a print help routine: prints all words at the specified level
       printlevel(const int level, const int width) const;
    const char * // returns the word contained in this branch
       word(void) const { return _word; }

    void         // writes the subtree out pre-order into the stream fp
       write_pre_order(FILE * const fp) const;
private:
    const char * // help routine for the remove() member function
       get_largest_word(void) const;

    char   * _word;    // the word that this branch contains
    branch * _left;    // the left subtree
    branch * _right;   // the right subtree
};


